#include <cppunit/config/SourcePrefix.h>
#include <cppunit/TestSuite.h>
#include <cppunit/TestResult.h>
#include <stdint.h>

CPPUNIT_NS_BEGIN

unsigned long TestSuite::disabledTests = 0;

/// Default constructor
TestSuite::TestSuite( std::string name )
    : TestComposite( name )
{
}


/// Destructor
TestSuite::~TestSuite()
{ 
  deleteContents(); 
}


/// Deletes all tests in the suite.
void 
TestSuite::deleteContents()
{
  int childCount = getChildTestCount();
  for ( int index =0; index < childCount; ++index )
    delete getChildTestAt( index );

  m_tests.clear();
}


/// Adds a test to the suite. 
void 
TestSuite::addTest( Test *test )
{ 
  m_tests.push_back( test ); 
}

bool
TestSuite::filter(const Filter& f)
{
    CppUnitVector<Test *> remaining;
    for (uint32_t i=0; i<m_tests.size(); ++i) {
        if (m_tests[i]->filter(f) || (m_tests[i]->getType() == Test::DISABLED)) {
            if (m_tests[i]->getType() == Test::DISABLED) {
                ++disabledTests;
            }
            delete m_tests[i];
            m_tests[i] = 0;
        } else {
            remaining.push_back(m_tests[i]);
        }
    }
    remaining.swap(m_tests);
    return false;
}

const CppUnitVector<Test *> &
TestSuite::getTests() const
{
  return m_tests;
}


int 
TestSuite::getChildTestCount() const
{
  return m_tests.size();
}


Test *
TestSuite::doGetChildTestAt( int index ) const
{
  return m_tests[index];
}


CPPUNIT_NS_END

