#include <cppunit/TestFailure.h>
#include <cppunit/TextTestProgressListener.h>
#include <cppunit/portability/Stream.h>
#include <cppunit/Test.h>
#include <sys/time.h>
#include <stdio.h>

namespace {
  uint64_t getTimeInMillis() {
      struct timeval t;
      gettimeofday(&t, 0);
      return (t.tv_sec * uint64_t(1000)) + (t.tv_usec / uint64_t(1000));
  }
  void reportEndOfTest(std::ostream& out, const std::string& result,
                       uint64_t& startTime, uint32_t nameLen)
  {
      uint64_t endTime = getTimeInMillis();
      uint64_t totalTime = endTime - startTime;
      startTime = 0;
      for (uint32_t i=nameLen; i<66; ++i) {
          out << " ";
      }
      out << " " << result << " " << totalTime << " ms";
  }
}

CPPUNIT_NS_BEGIN

std::unique_ptr<TextTestProgressListener::TestStartHook> TextTestProgressListener::startHook;
bool TextTestProgressListener::verboseProgress = false;
uint64_t TextTestProgressListener::startTime = 0;
uint32_t TextTestProgressListener::nameLen = 0;

unsigned long TextTestProgressListener::ignoredTests = 0;

TextTestProgressListener::TextTestProgressListener()
{
}


TextTestProgressListener::~TextTestProgressListener()
{
}


void 
TextTestProgressListener::startTest( Test *test )
{
  if (startTime != 0 && verboseProgress) {
      reportEndOfTest(stdCOut(), "OK", startTime, nameLen);
      if (startHook.get() != 0) startHook->stoppedTest();
  }
  startTime = getTimeInMillis();
  if (verboseProgress) {
    std::string name = test->getName();
    if (startHook.get() != 0) startHook->startedTest(name);
    std::string::size_type index = name.rfind("::");
    if (index != std::string::npos) {
        index = name.rfind("::", index-1);
        if (index != std::string::npos) {
            name = name.substr(index+2);
        }
    }
    nameLen = name.size();
    stdCOut() << "\n" << name;
  } else {
  stdCOut() << ".";
  }
  stdCOut().flush();
}


void 
TextTestProgressListener::addFailure( const TestFailure &failure )
{
  if (verboseProgress) {
    reportEndOfTest(stdCOut(), failure.isError() ? "E" : "F", startTime,
                    nameLen);
  } else {
  stdCOut() << ( failure.isError() ? "E" : "F" );
  }
  if (failure.failedTest()->getType() == Test::IGNORED) {
    ++ignoredTests;
    stdCOut() << " (ignored)";
  }
  stdCOut().flush();
}


void 
TextTestProgressListener::endTestRun( Test *test, 
                                      TestResult *eventManager )
{
  if (verboseProgress && startTime != 0) {
    reportEndOfTest(stdCOut(), "OK", startTime, nameLen);
  }
  stdCOut()  <<  "\n";
  stdCOut().flush();
}


CPPUNIT_NS_END

